@@ -784,7 +784,8 @@ static void prism2_tx_timeout(struct net_device *dev, unsigned int txqueue)
 #if LINUX_VERSION_IS_LESS(5,6,0)
 /* Just declare it here to keep sparse happy */
 void bp_prism2_tx_timeout(struct net_device *dev);
-void bp_prism2_tx_timeout(struct net_device *dev) {
+void bp_prism2_tx_timeout(struct net_device *dev)
+{
 	prism2_tx_timeout(dev, 0);
 }
 EXPORT_SYMBOL_GPL(bp_prism2_tx_timeout);
