@@ -1045,7 +1045,8 @@ mwifiex_tx_timeout(struct net_device *dev, unsigned int txqueue)
 #if LINUX_VERSION_IS_LESS(5,6,0)
 /* Just declare it here to keep sparse happy */
 void bp_mwifiex_tx_timeout(struct net_device *dev);
-void bp_mwifiex_tx_timeout(struct net_device *dev) {
+void bp_mwifiex_tx_timeout(struct net_device *dev)
+{
 	mwifiex_tx_timeout(dev, 0);
 }
 EXPORT_SYMBOL_GPL(bp_mwifiex_tx_timeout);
